@@ -5,40 +5,43 @@
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
  * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * version 2 of the License, or (at your option) any later version.
  * 
  * This software is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
+#include "b_config.h"
+
 #include <string.h>
 #include <stdlib.h>
-#include <net/if.h>
+#include <unistd.h>
+#include <assert.h>
+#include <errno.h>
+#include <fcntl.h>
+#include <netdb.h>  /* getnameinfo */
+#include <poll.h>
+#include <arpa/inet.h>
 #include <asm/types.h>
 #include <linux/rtnetlink.h>
-#include <arpa/inet.h>
+#include <net/if.h>
 #include <netinet/in.h>
 #include <netinet/tcp.h>
-#include <assert.h>
-#include <unistd.h>
-#include <fcntl.h>
-#include <errno.h>
-#include <poll.h>
-#include "b_config.h"
+#include <sys/socket.h>  /* getnameinfo */
+#include "attr.h"
+#include "auth.h"
 #include "booth.h"
+#include "config.h"
 #include "inline-fn.h"
 #include "log.h"
-#include "config.h"
 #include "ticket.h"
 #include "transport.h"
-#include "auth.h"
-#include "attr.h"
 
 #define BOOTH_IPADDR_LEN	(sizeof(struct in6_addr))
 
@@ -50,7 +53,12 @@
 
 struct booth_site *local = NULL;
 
-
+/* function to be called when handling booth-group-internal messages;
+ * it's expected to return 0 to indicate success, negative integer
+ * to indicate silent (or possibly already complained about) error,
+ * or positive integer to indicate sender's ID that will then be
+ * emitted in the error log message together with the real source
+ * address if this is available */
 static int (*deliver_fn) (void *msg, int msglen);
 
 
@@ -100,10 +108,7 @@ static int find_address(unsigned char ipaddr[BOOTH_IPADDR_LEN],
 			if (ipaddr[matched] != n_a[matched])
 				break;
 
-
 		if (matched == node->addrlen) {
-			/* Exact match. */
-exact_match:
 			*address_bits_matched = matched * 8;
 			*me = node;
 			did_match = EXACT_MATCH;
@@ -119,8 +124,6 @@ exact_match:
 			continue;
 		if (matched * 8 < *address_bits_matched)
 			continue;
-		if (!bits_left)
-			goto exact_match;
 
 		node_bits = n_a[bytes];
 		ip_bits = ipaddr[bytes];
@@ -167,7 +170,7 @@ int _find_myself(int family, struct booth_site **mep, int fuzzy_allowed)
 		return 0;
 	}
 
-	setsockopt(fd, SOL_SOCKET, SO_RCVBUF, &rcvbuf, sizeof(rcvbuf));
+	(void)setsockopt(fd, SOL_SOCKET, SO_RCVBUF, &rcvbuf, sizeof(rcvbuf));
 
 	memset(&nladdr, 0, sizeof(nladdr));
 	nladdr.nl_family = AF_NETLINK;
@@ -205,17 +208,16 @@ int _find_myself(int family, struct booth_site **mep, int fuzzy_allowed)
 			return 0;
 		}
 
-		h = (struct nlmsghdr *)rcvbuf;
-		if (h->nlmsg_type == NLMSG_DONE)
-			break;
+		for (h = (struct nlmsghdr *)rcvbuf; NLMSG_OK(h, status); h = NLMSG_NEXT(h, status)) {
+			if (h->nlmsg_type == NLMSG_DONE)
+				goto out;
 
-		if (h->nlmsg_type == NLMSG_ERROR) {
-			close(fd);
-			log_error("netlink socket recvmsg error");
-			return 0;
-		}
+			if (h->nlmsg_type == NLMSG_ERROR) {
+				close(fd);
+				log_error("netlink socket recvmsg error");
+				return 0;
+			}
 
-		while (NLMSG_OK(h, status)) {
 			if (h->nlmsg_type == RTM_NEWADDR) {
 				struct ifaddrmsg *ifa = NLMSG_DATA(h);
 				struct rtattr *tb[IFA_MAX+1];
@@ -235,21 +237,43 @@ int _find_myself(int family, struct booth_site **mep, int fuzzy_allowed)
 							BOOTH_IPADDR_LEN);
 				}
 
-				/* First try with exact addresses, then optionally with subnet matching. */
+				/* Try to find the exact address or the address with subnet matching.
+				 * The function find_address will be called for each address received
+				 * from NLMSG_DATA above.
+				 * The exact match will be preferred. If no exact match is found,
+				 * the function find_address will try to return another, most similar
+				 * address (with the longest possible number of same bytes). */
 				if (ifa->ifa_prefixlen > address_bits_matched) {
 					find_address(ipaddr,
 							ifa->ifa_family, ifa->ifa_prefixlen,
 							fuzzy_allowed, &me, &address_bits_matched);
+
 					if (me) {
 						log_debug("found myself at %s (%d bits matched)",
 								site_string(me), address_bits_matched);
 					}
 				}
+				/* If the previous NLMSG_DATA calls have already allowed us
+				 * to find an address with address_bits_matched matching bits,
+				 * then no other better non-exact address can be found.
+				 * But we can still try to find an exact match, so let us
+				 * call the function find_address with disabled searching of
+				 * similar addresses (fuzzy_allowed == 0) */
+				else if (ifa->ifa_prefixlen == address_bits_matched) {
+					find_address(ipaddr,
+							ifa->ifa_family, ifa->ifa_prefixlen,
+							0 /* fuzzy_allowed */, &me, &address_bits_matched);
+
+					if (me) {
+						log_debug("found myself at %s (exact match)",
+								site_string(me));
+					}
+				}
 			}
-			h = NLMSG_NEXT(h, status);
 		}
 	}
 
+out:
 	close(fd);
 
 	if (!me)
@@ -370,7 +394,7 @@ int read_client(struct client *req_cl)
 	header = (struct boothc_header *)msg;
 
 	/* update len if we read enough */
-	if (req_cl->offset >= sizeof(header)) {
+	if (req_cl->offset >= sizeof(*header)) {
 		len = min(ntohl(header->length), MAX_MSG_LEN);
 	}
 
@@ -384,7 +408,7 @@ int read_client(struct client *req_cl)
 	req_cl->offset += rv;
 
 	/* update len if we read enough */
-	if (req_cl->offset >= sizeof(header)) {
+	if (req_cl->offset >= sizeof(*header)) {
 		len = min(ntohl(header->length), MAX_MSG_LEN);
 	}
 
@@ -490,10 +514,15 @@ static void process_tcp_listener(int ci)
 			  fd, errno);
 		return;
 	}
-	setsockopt(fd, IPPROTO_TCP, TCP_NODELAY, (char *)&one, sizeof(one));
+	(void)setsockopt(fd, IPPROTO_TCP, TCP_NODELAY, (char *)&one, sizeof(one));
 
 	flags = fcntl(fd, F_GETFL, 0);
-	fcntl(fd, F_SETFL, flags | O_NONBLOCK);
+	if (fcntl(fd, F_SETFL, flags | O_NONBLOCK) == -1) {
+		log_error("process_tcp_listener: fcntl O_NONBLOCK error %d %d",
+			  fd, errno);
+		(void)close(fd);
+		return;
+	}
 
 	i = client_add(fd, clients[ci].transport,
 			process_connection, NULL);
@@ -514,6 +543,7 @@ int setup_tcp_listener(int test_only)
 
 	rv = setsockopt(s, SOL_SOCKET, SO_REUSEADDR, (char *)&one, sizeof(one));
 	if (rv == -1) {
+		close(s);
 		log_error("failed to set the SO_REUSEADDR option");
 		return rv;
 	}
@@ -526,12 +556,14 @@ int setup_tcp_listener(int test_only)
 	}
 
 	if (rv == -1) {
+		close(s);
 		log_error("failed to bind socket %s", strerror(errno));
 		return rv;
 	}
 
 	rv = listen(s, 5);
 	if (rv == -1) {
+		close(s);
 		log_error("failed to listen on socket %s", strerror(errno));
 		return rv;
 	}
@@ -565,7 +597,10 @@ static int connect_nonb(int sockfd, const struct sockaddr *saptr,
 	struct timeval	tval;
 
 	flags = fcntl(sockfd, F_GETFL, 0);
-	fcntl(sockfd, F_SETFL, flags | O_NONBLOCK);
+	if (fcntl(sockfd, F_SETFL, flags | O_NONBLOCK) == -1) {
+		log_error("fcntl: Can't set sockfd to nonblocking mode");
+		return -1;
+	}
 
 	error = 0;
 	if ( (n = connect(sockfd, saptr, salen)) < 0)
@@ -600,7 +635,11 @@ static int connect_nonb(int sockfd, const struct sockaddr *saptr,
 	}
 
 done:
-	fcntl(sockfd, F_SETFL, flags);	/* restore file status flags */
+	/* restore file status flags */
+	if (fcntl(sockfd, F_SETFL, flags) == -1) {
+		log_error("fcntl: Can't restore sockfd flags");
+		return -1;
+	}
 
 	if (error) {
 		/* leave outside function to close */
@@ -765,8 +804,8 @@ static void process_recv(int ci)
 	struct sockaddr_storage sa;
 	int rv;
 	socklen_t sa_len;
-	/* beware, the buffer needs to be large enought to accept a
-	 * packet */
+	/* beware, the buffer needs to be large enough to accept
+	 * a packet */
 	char buffer[MAX_MSG_LEN];
 	/* Used for unit tests */
 	struct boothc_ticket_msg *msg;
@@ -781,7 +820,15 @@ static void process_recv(int ci)
 	if (rv == -1)
 		return;
 
-	deliver_fn((void*)msg, rv);
+	rv = deliver_fn((void*)msg, rv);
+	if (rv > 0) {
+		if (getnameinfo((struct sockaddr *)&sa, sa_len,
+				buffer, sizeof(buffer), NULL, 0,
+				NI_NUMERICHOST) == 0)
+			log_error("unknown sender: %08x (real: %s)", rv, buffer);
+		else
+			log_error("unknown sender: %08x", rv);
+	}
 }
 
 static int booth_udp_init(void *f)
@@ -930,7 +977,7 @@ const struct booth_transport booth_transport[TRANSPORT_ENTRIES] = {
 int add_hmac(void *data, int len)
 {
 	int rv = 0;
-#if HAVE_LIBGCRYPT || HAVE_LIBMHASH
+#if HAVE_LIBGNUTLS || HAVE_LIBGCRYPT || HAVE_LIBMHASH
 	int payload_len;
 	struct hmac *hp;
 
@@ -950,7 +997,7 @@ int add_hmac(void *data, int len)
 	return rv;
 }
 
-#if HAVE_LIBGCRYPT || HAVE_LIBMHASH
+#if HAVE_LIBGNUTLS || HAVE_LIBGCRYPT || HAVE_LIBMHASH
 
 /* TODO: we need some client identification for logging */
 #define peer_string(p) (p ? site_string(p) : "client")
@@ -1004,7 +1051,7 @@ accept:
 int check_auth(struct booth_site *from, void *buf, int len)
 {
 	int rv = 0;
-#if HAVE_LIBGCRYPT || HAVE_LIBMHASH
+#if HAVE_LIBGNUTLS || HAVE_LIBGCRYPT || HAVE_LIBMHASH
 	int payload_len;
 	struct hmac *hp;
 
@@ -1053,7 +1100,7 @@ int send_header_plus(int fd, struct boothc_hdr_msg *msg, void *data, int len)
 	return rv;
 }
 
-/* UDP message receiver. */
+/* UDP message receiver (see also deliver_fn declaration's comment) */
 int message_recv(void *msg, int msglen)
 {
 	uint32_t from;
@@ -1063,9 +1110,11 @@ int message_recv(void *msg, int msglen)
 	header = (struct boothc_header *)msg;
 
 	from = ntohl(header->from);
-	if (!find_site_by_id(from, &source) || !source) {
-		log_error("unknown sender: %08x", from);
-		return -1;
+	if (!find_site_by_id(from, &source)) {
+		/* caller knows the actual source address, pass
+		   the (assuredly) positive number and let it report */
+		from = from ? from : ~from;  /* avoid 0 (success) */
+		return from & (~0U >> 1);  /* avoid negative (error code} */
 	}
 
 	time(&source->last_recv);
