@@ -1,13 +1,11 @@
-#!/usr/bin/python
-
 from boothtestenv import BoothTestEnvironment
 from boothrunner  import BoothRunner
 
 class ClientTestEnvironment(BoothTestEnvironment):
     mode = 'client'
 
-    def run_booth(self, config_text=None, config_file=None, lock_file=True, args=[],
-                  expected_exitcode=0, debug=False):
+    def run_booth(self, config_text=None, config_file=None, lock_file=True,
+                  args=(), expected_exitcode=0, debug=False):
         '''
         Runs boothd.
 
@@ -18,7 +16,7 @@ class ClientTestEnvironment(BoothTestEnvironment):
 
         runner = BoothRunner(self.boothd_path, self.mode, args)
         runner.show_args()
-        (pid, return_code, stdout, stderr) = runner.run()
+        (pid, return_code, stdout, stderr) = runner.run(expected_exitcode)
         self.check_return_code(pid, return_code, expected_exitcode)
 
         return (pid, return_code, stdout, stderr, runner)
